"""
===========================================
显示估计器和复杂的管道
===========================================

这个例子展示了显示估计器和管道的不同方法。
"""

from sklearn.compose import make_column_transformer
from sklearn.impute import SimpleImputer
from sklearn.linear_model import LogisticRegression
from sklearn.pipeline import make_pipeline
from sklearn.preprocessing import OneHotEncoder, StandardScaler

# %%
# 紧凑文本表示
# ---------------------------
#
# 估计器在作为字符串显示时，只会显示已设置为非默认值的参数。这减少了视觉噪音，并使比较实例时更容易发现差异。

lr = LogisticRegression(penalty="l1")
print(lr)

# %%
# 丰富的HTML表示
# ----------------
# 在笔记本中，估计器和管道将使用丰富的HTML表示。这对于总结管道和其他复合估计器的结构特别有用，并具有交互性以提供详细信息。点击下面的示例图像以展开管道元素。有关如何使用此功能，请参见 :ref:`visualizing_composite_estimators` 。


num_proc = make_pipeline(SimpleImputer(strategy="median"), StandardScaler())

cat_proc = make_pipeline(
    SimpleImputer(strategy="constant", fill_value="missing"),
    OneHotEncoder(handle_unknown="ignore"),
)

preprocessor = make_column_transformer(
    (num_proc, ("feat1", "feat3")), (cat_proc, ("feat0", "feat2"))
)

clf = make_pipeline(preprocessor, LogisticRegression())
clf
