"""
==========================
绘制验证曲线
==========================

在此图中，您可以看到支持向量机（SVM）在不同核参数 gamma 值下的训练分数和验证分数。对于非常低的 gamma 值，您会发现训练分数和验证分数都很低。这被称为欠拟合。中等的 gamma 值将导致两个分数都很高，即分类器表现相当好。如果 gamma 值过高，分类器将会过拟合，这意味着训练分数很好但验证分数很差。

"""

import matplotlib.pyplot as plt
import numpy as np

from sklearn.datasets import load_digits
from sklearn.model_selection import ValidationCurveDisplay
from sklearn.svm import SVC

X, y = load_digits(return_X_y=True)
subset_mask = np.isin(y, [1, 2])  # binary classification: 1 vs 2
X, y = X[subset_mask], y[subset_mask]

disp = ValidationCurveDisplay.from_estimator(
    SVC(),
    X,
    y,
    param_name="gamma",
    param_range=np.logspace(-6, -1, 5),
    score_type="both",
    n_jobs=2,
    score_name="Accuracy",
)
disp.ax_.set_title("Validation Curve for SVM with an RBF kernel")
disp.ax_.set_xlabel(r"gamma (inverse radius of the RBF kernel)")
disp.ax_.set_ylim(0.0, 1.1)
plt.show()
