"""
======================
带有自定义核函数的SVM
======================

使用支持向量机对样本进行分类的简单示例。它将绘制决策面和支持向量。

"""

import matplotlib.pyplot as plt
import numpy as np

from sklearn import datasets, svm
from sklearn.inspection import DecisionBoundaryDisplay

# 导入一些数据来玩玩
iris = datasets.load_iris()
X = iris.data[:, :2]  # we only take the first two features. We could
# 通过使用二维数据集来避免这种丑陋的切片
Y = iris.target


def my_kernel(X, Y):
    """我们创建一个自定义核函数：

             (2  0)
k(X, Y) = X  (    ) Y.T
             (0  1)
"""
    M = np.array([[2, 0], [0, 1.0]])
    return np.dot(np.dot(X, M), Y.T)


h = 0.02  # step size in the mesh

# 我们创建一个SVM实例并拟合我们的数据。
clf = svm.SVC(kernel=my_kernel)
clf.fit(X, Y)

ax = plt.gca()
DecisionBoundaryDisplay.from_estimator(
    clf,
    X,
    cmap=plt.cm.Paired,
    ax=ax,
    response_method="predict",
    plot_method="pcolormesh",
    shading="auto",
)

# 还要绘制训练点
plt.scatter(X[:, 0], X[:, 1], c=Y, cmap=plt.cm.Paired, edgecolors="k")
plt.title("3-Class classification using Support Vector Machine with custom kernel")
plt.axis("tight")
plt.show()
